/**
 * Converts the given dsid to a dsuid
 * @param {String} dSID
 * @returns {String} dSUID
 */
function dSID2dSUID(dSID) {
    try {
        //copy the first 6 bytes (12 characters to the new dsuid
        var dSUID = dSID.slice(0, 12);
        //insert 4 bytes of 0
        dSUID += '00000000';
        //append the remaining 6 bytes (12 chars) of the dsid
        dSUID += dSID.slice(12, 24);
        //finaly add 1 byte of 0
        dSUID += '00';
        return dSUID;
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::dSID2dSUID ' + e);
        return '';
    }
}


/**
 * Converts the trigger entries from dsid to dsuid
 * @returns {boolean} true if property tree changed, false otherwize
 */
function convert2DSUID() {
    var fMigrate = false;
    try {
        lInfo.logln('convert2DSUID start');
        var iIndex, jIndex;
        var dSID, dSUID;
        var baseNode = Property.getNode('mddevices');
        if (baseNode != null) {
            var oArrayNode = baseNode.getChildren();
            for (iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
                var oChildNode = oArrayNode[iIndex];
                if (oChildNode.getChild('deviceID') != null) {
                    dSID = oChildNode.getChild('deviceID').getValue();
                    dSUID = dSID2dSUID(dSID);
                    Property.setProperty(oChildNode.getChild('deviceID').getPath(), dSUID);
                    Property.setFlag(oChildNode.getChild('deviceID').getPath(), 'ARCHIVE', true); 
                }
            }
        }
    } catch(e) {
        lWarn.logln('Exception in backward-compatibility::convert2DSUID: ' + e);
    }
    return fMigrate;
}

/**
 * Compares version strings
 * @param {string} strV1 version string 1 to compare 
 * @param {string} strV2 version string 1 to compare
 * @returns {Number|undefined} return   number < 0  if strV1 < strV2
 *                                          0       if strV1 = strV2
 *                                      number >0   if strV1 < strV2
 */
function compVersions(strV1, strV2) {
    try {
        var nRes = 0;
        var parts1 = strV1.split('.');
        var parts2 = strV2.split('.');
        var nLen = Math.max(parts1.length, parts2.length);

        for (var i = 0; i < nLen; i++) {
          var nP1 = (i < parts1.length) ? parseInt(parts1[i], 10) : 0;
          var nP2 = (i < parts2.length) ? parseInt(parts2[i], 10) : 0;

          if (isNaN(nP1)) { nP1 = 0; }
          if (isNaN(nP2)) { nP2 = 0; }

          if (nP1 !== nP2) {
            nRes = (nP1 > nP2) ? 1 : -1;
            break;
          }
        }
        return nRes;
    } catch(e) {
        lWarn.logln('Exception in backward-compatibility::compVersions ' + e);
        return 0;       
    }
}

/**
 * convert property tree entries if needed
 * @returns {undefined}
 */
function convertPropertyTreeEntries()
{
    var newVersion = '1.0.0';
    var fMigrated = false;
    try {      
        var propVersion = Property.getNode('version');
        var currentVersion = (propVersion != null) ? propVersion.getValue() : '0.0.0';
        lInfo.logln('check if conversions from version "' + currentVersion + '" to version "' + newVersion + '" are needed');    
        if (compVersions(currentVersion, '1.0.0') < 0) {
            fMigrated = convert2DSUID() || fMigrated;   
        } else {
            lInfo.logln('no conversion needed');    
        }
        Property.setProperty('version', newVersion);
        Property.setFlag('version', 'ARCHIVE', true);
        lInfo.logln("motion detector running (Version " + newVersion + ")");
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::convertPropertyTreeEntries ' + e);
    }
    if (fMigrated) {
        Property.store();
    }
}

